/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2023 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.1.0                                        *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_RL78_1.c for Renesas RL78 CPU
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#ifdef __ICCRL78__
  #include <ior5f100LE.h>
  #include <ior5f100LE_ext.h>
#elif defined (__GNUC__)
  #include "iodefine.h"
  #include "iodefine_ext.h"
#elif defined (__CCRL__)
  #include "iodefine.h"
#endif

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#define OS_TIMER_FREQ  (20000000uL)
#define OS_TICK_FREQ   (1000u)
#define OS_INT_FREQ    (OS_TICK_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_DISABLED
#endif

/*********************************************************************
*
*       CPU configuration,
*       Option bytes and security code
*/
#ifdef __ICCRL78__
  #pragma diag_suppress=Ta026

  #pragma location = "OPTBYTE"
  __root const unsigned char opbytes[4]= {
    0x6Eu, 0xFFu, 0x8Eu, 0x85u
  };

  //
  // Security Byte Definition
  //
  #pragma location = "SECUID"
  __root const unsigned char secuid[10]= {
    0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu, 0xffu
  };

  #define _NOP() __no_operation()

#elif defined (__GNUC__)
  //
  // Please ensure that sections ".option_bytes" and ".security_id" are defined in linker
  // and ensure that the option bytes are defined in vector_table.c
  //
  #define _NOP() __asm("nop")
#elif defined (__CCRL__)
  //
  // Please ensure the Security ID is defined in linker settings
  // and ensure the ".option_byte" section is defined in cstart.asm
  //
  #define _NOP() __nop()
#else
  #error "Undefined Compiler, Optionbytes have to be defined !!!"
#endif // Compiler specific option byte definition

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value
*
*  Return value
*    Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return TCR00;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set
*
*  Return value
*    == 0; Interrupt pending flag not set
*    != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return TMIF00;
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*  Function description
*    This is the code that gets called when the processor receives a
*    INTTM00 exception. TM00 is used as OS timer tick.
*/
#ifdef __ICCRL78__
  #define OS_TICKVECT   INTTM00_vect
  #pragma vector=OS_TICKVECT
  static __interrupt __v1_call void OS_ISR_Tick(void) {
#elif defined (__CCRL__)
  #pragma interrupt OS_ISR_Tick (vect=INTTM00)
  static void OS_ISR_Tick (void) {
#elif defined (__GNUC__)
  void __attribute__ ((interrupt)) OS_ISR_Tick(void) {
#endif
  OS_INT_EnterNestable();
  OS_INT_EnterIntStack();
  OS_TICK_Handle();
  OS_INT_LeaveIntStack();
  OS_INT_LeaveNestable();
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/
/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware (timer) required for embOS to run.
*    May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  //
  // Initially disable interrupts
  //
  OS_INT_IncDI();
  //
  // Enable Timer array unit clock first
  // This is required before programming any other timer array sfr
  //
  // No need to initialize timer hardware when using simulator
  TMIF00 = 0;  // Clear interrupt request Bit Timer0
  TMMK00 = 0;  // Enable Timer0 Interrupt
  //
  // Inform embOS about the timer settings
  //
  {
    OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_DOWNCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};
    OS_TIME_ConfigSysTimer(&SysTimerConfig);
  }
  //
  // Initialize the optional communication for embOSView
  //
#if (OS_VIEW_IFSELECT != OS_VIEW_DISABLED)
  OS_COM_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This is basically the "core" of the idle loop.
*    This core loop can be changed, but:
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved. However, a simple program loop can be programmed
*    (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {    // Idle loop: No task is ready to execute
  while (1) {           // Nothing to do ... wait for interrupt
    #if (OS_DEBUG == 0)
                        // Switch CPU into sleep mode
    #endif
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/
#if (OS_VIEW_IFSELECT == OS_VIEW_DISABLED)

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Dummy routine.
*/
void OS_COM_Send1(OS_U8 c) {
  OS_USE_PARA(c);          // Avoid compiler warning
  OS_COM_ClearTxActive();  // Let embOS know that Tx is not busy
}

#else
  #error "Selected embOSView interface is currently not supported."
#endif

#if defined(__ICCRL78__)
/*********************************************************************
*
*       __low_level_init()
*       Perform hardware initialization which has to be done
*       before memory is initialized
*       Because this function is called, before variables are initialized,
*       no functions that rely on any variables can be performed
*/
#ifdef __cplusplus
extern "C" {
#endif
int __low_level_init(void);    // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
int __low_level_init(void) {
  return 1;           // Always return 1 to enable segment initialization
}
#endif  // __ICCRL78__

/****** End Of File *************************************************/
